/* BdfServer_JsonProducers - Copyright (c) 2022-2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.jsonproducers.selection;

import fr.exemole.bdfserver.api.instruction.BdfCommandResult;
import fr.exemole.bdfserver.api.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.api.instruction.BdfParameters;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.interaction.domains.SelectionDomain;
import fr.exemole.bdfserver.jsonproducers.ResultJsonProducer;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.instruction.BdfErrors;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.subsettree.TreeUtils;
import java.util.Comparator;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.Fiches;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.FichothequeQueries;
import net.fichotheque.selection.SelectionContext;
import net.fichotheque.tools.corpus.FichesBuilder;
import net.fichotheque.tools.permission.PermissionUtils;
import net.fichotheque.tools.selection.FichothequeQueriesBuilder;
import net.fichotheque.tools.selection.SelectionDOMUtils;
import net.fichotheque.utils.Comparators;
import net.fichotheque.utils.SelectionUtils;
import net.fichotheque.utils.selection.FicheSelectorBuilder;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.xml.DOMUtils;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public final class SelectionJsonProducerFactory {

    private SelectionJsonProducerFactory() {

    }

    public static JsonProducer getJsonProducer(OutputParameters parameters) throws ErrorMessageException {
        String json = parameters.getOutput();
        BdfCommandResult bdfCommandResult = parameters.getBdfCommandResult();
        ResultJsonProducer jsonProducer = new ResultJsonProducer(bdfCommandResult, parameters.getMessageLocalisation());
        switch (json) {
            case SelectionDomain.FQL_JSON: {
                boolean done = false;
                if (bdfCommandResult != null) {
                    FichothequeQueries fichothequeQueries = (FichothequeQueries) bdfCommandResult.getResultObject(BdfInstructionConstants.FICHOTHEQUEQUERIES_OBJ);
                    if (fichothequeQueries != null) {
                        done = true;
                        jsonProducer.add(new FqlJsonProperty(parameters.getFichotheque(), fichothequeQueries));
                    }
                }
                if (done) {
                    return jsonProducer;
                } else {
                    throw BdfErrors.missingCommandResultJson(SelectionDomain.FQL_JSON);
                }
            }
            case SelectionDomain.FICHES_JSON: {
                Fiches fiches = getFiches(parameters, parameters.getRequestMap());
                jsonProducer.add(new FichesJsonProperty(parameters.getPermissionSummary(), fiches, BdfInstructionUtils.getCellEngine(parameters, parameters.getRequestMap()), parameters.getWorkingLang()));
                return jsonProducer;
            }
            default:
                return null;
        }

    }

    private static Fiches getFiches(BdfParameters bdfParameters, RequestMap requestMap) throws ErrorMessageException {
        String xml = BdfInstructionUtils.getMandatoryParameter(requestMap, SelectionDomain.XML_PARAMNAME);
        org.w3c.dom.Document document;
        try {
            document = DOMUtils.parseDocument(xml);
        } catch (SAXException sae) {
            throw BdfErrors.error("_ error.exception.xml.sax", sae.getMessage());
        }
        SelectionContext selectionContext = BdfServerUtils.initSelectionContextBuilder(bdfParameters)
                .setSubsetAccessPredicate(bdfParameters.getPermissionSummary().getSubsetAccessPredicate())
                .setFichePredicate(PermissionUtils.getFichePredicate(bdfParameters.getPermissionSummary()))
                .toSelectionContext();
        FicheSelectorBuilder builder = FicheSelectorBuilder.init(selectionContext);
        Element root = document.getDocumentElement();
        switch (root.getTagName()) {
            case "fiche-query":
            case "fiche-select":
                FicheQuery ficheQuery = SelectionDOMUtils.getFicheConditionEntry(bdfParameters.getFichotheque(), root).getFicheQuery();
                builder.add(ficheQuery);
                break;
            default:
                FichothequeQueriesBuilder fichothequeQueriesBuilder = new FichothequeQueriesBuilder();
                DOMUtils.readChildren(root, SelectionDOMUtils.getQueryElementConsumer(bdfParameters.getFichotheque(), fichothequeQueriesBuilder));
                FichothequeQueries fichothequeQueries = fichothequeQueriesBuilder.toFichothequeQueries();
                if (fichothequeQueries.getFicheQueryList().isEmpty()) {
                    throw BdfErrors.unsupportedParameterValue(SelectionDomain.XML_PARAMNAME, "Missing fiche-query elements");
                }
                for (FicheQuery fq : fichothequeQueries.getFicheQueryList()) {
                    builder.add(fq);
                }
        }
        return FichesBuilder.init(getComparator(selectionContext, requestMap))
                .initSubsetKeyOrder(TreeUtils.getCorpusKeyList(bdfParameters.getBdfServer()))
                .populate(builder.toFicheSelector())
                .toFiches();
    }

    private static Comparator<FicheMeta> getComparator(SelectionContext selectionContext, RequestMap requestMap) throws ErrorMessageException {
        String sort = requestMap.getParameter(SelectionDomain.SORT_PARAMNAME);
        if ((sort == null) || (sort.isEmpty())) {
            return Comparators.FICHEID_ASC;
        }
        try {
            sort = SelectionUtils.checkSortType(sort);
            return Comparators.getComparator(sort, selectionContext.getWorkingLang());
        } catch (IllegalArgumentException iae) {
            throw BdfErrors.error("_ error.unknown.parametervalue", SelectionDomain.SORT_PARAMNAME, sort);
        }
    }

}
