/* FichothequeLib_Tools - Copyright (c) 2013-2018 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.importation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheChange;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.fiche.ContentChecker;
import net.fichotheque.importation.CorpusImport;
import net.fichotheque.importation.LiensImport;
import net.fichotheque.tools.corpus.FicheChangeBuilder;
import net.mapeadores.util.attr.AttributeChange;
import net.mapeadores.util.attr.AttributeChangeBuilder;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.primitives.FuzzyDate;


/**
 *
 * @author Vincent Calame
 */
public class CorpusImportBuilder {

    private final ContentChecker contentChecker;
    private final InternalCorpusImport corpusImport;
    private final Map<Integer, CorpusImport.FicheImport> ficheMap = new HashMap<Integer, CorpusImport.FicheImport>();

    private CorpusImportBuilder(ContentChecker contentChecker, Corpus corpus, String type) {
        this.contentChecker = contentChecker;
        this.corpusImport = new InternalCorpusImport(corpus, type);
    }

    public ContentChecker getContentChecker() {
        return contentChecker;
    }

    public String getType() {
        return corpusImport.getType();
    }

    public Corpus getCorpus() {
        return corpusImport.getCorpus();
    }

    public void addRemove(FicheMeta ficheMeta) {
        if (!getType().equals(CorpusImport.REMOVE_TYPE)) {
            throw new UnsupportedOperationException("type != CorpusImport.REMOVE_TYPE");
        }
        int ficheid = ficheMeta.getId();
        if (!ficheMap.containsKey(ficheid)) {
            CorpusImport.FicheImport ficheImport = new InternalFicheImport(ficheMeta);
            ficheMap.put(ficheid, ficheImport);
            corpusImport.add(ficheImport);
        }
    }

    public void addChange(FicheMeta ficheMeta, FicheChangeBuilder ficheChangeBuilder, AttributeChangeBuilder attributeChangeBuilder, LiensImportBuilder liensImportBuilder, FuzzyDate creationDate) {
        if (!getType().equals(CorpusImport.CHANGE_TYPE)) {
            throw new UnsupportedOperationException("type != CorpusImport.CHANGE_TYPE");
        }
        int ficheid = ficheMeta.getId();
        InternalChangeFicheImport changeFicheImport = (InternalChangeFicheImport) ficheMap.get(ficheid);
        if (changeFicheImport == null) {
            changeFicheImport = new InternalChangeFicheImport(ficheMeta, ficheChangeBuilder, attributeChangeBuilder, liensImportBuilder, creationDate);
            ficheMap.put(ficheid, changeFicheImport);
            corpusImport.add(changeFicheImport);
        } else {
            changeFicheImport.merge(ficheChangeBuilder, attributeChangeBuilder, liensImportBuilder, creationDate);
        }
    }

    public void addCreation(int newId, FicheChangeBuilder ficheChangeBuilder, AttributeChangeBuilder attributeChangeBuilder, LiensImportBuilder liensImportBuilder, FuzzyDate creationDate) {
        if (!getType().equals(CorpusImport.CREATION_TYPE)) {
            throw new UnsupportedOperationException("type != CorpusImport.CHANGE_TYPE");
        }
        if (newId > 0) {
            InternalCreationFicheImport creationFicheImport = (InternalCreationFicheImport) ficheMap.get(newId);
            if (creationFicheImport == null) {
                creationFicheImport = new InternalCreationFicheImport(newId, ficheChangeBuilder, attributeChangeBuilder, liensImportBuilder, creationDate);
                ficheMap.put(newId, creationFicheImport);
                corpusImport.add(creationFicheImport);
            } else {
                creationFicheImport.merge(ficheChangeBuilder, attributeChangeBuilder, liensImportBuilder, creationDate);
            }
        } else {
            corpusImport.add(new InternalCreationFicheImport(-1, ficheChangeBuilder, attributeChangeBuilder, liensImportBuilder, creationDate));
        }
    }

    public static CorpusImportBuilder newInstance(ContentChecker contentChecker, Corpus corpus, String type) {
        if (type.equals(CorpusImport.CHANGE_TYPE)) {
            return newChangeInstance(contentChecker, corpus);
        }
        if (type.equals(CorpusImport.CREATION_TYPE)) {
            return newCreationInstance(contentChecker, corpus);
        }
        if (type.equals(CorpusImport.REMOVE_TYPE)) {
            return newRemoveInstance(contentChecker, corpus);
        }
        throw new IllegalArgumentException();
    }

    public static CorpusImportBuilder newCreationInstance(ContentChecker contentChecker, Corpus corpus) {
        return new CorpusImportBuilder(contentChecker, corpus, CorpusImport.CREATION_TYPE);
    }

    public static CorpusImportBuilder newChangeInstance(ContentChecker contentChecker, Corpus corpus) {
        return new CorpusImportBuilder(contentChecker, corpus, CorpusImport.CHANGE_TYPE);
    }

    public static CorpusImportBuilder newRemoveInstance(ContentChecker contentChecker, Corpus corpus) {
        return new CorpusImportBuilder(contentChecker, corpus, CorpusImport.REMOVE_TYPE);
    }

    public CorpusImport toCorpusImport() {
        return corpusImport;
    }


    private static class InternalCorpusImport implements CorpusImport {

        private final Corpus thesaurus;
        private final String type;
        private final List<FicheImport> motcleImportList = new ArrayList<FicheImport>();

        private InternalCorpusImport(Corpus thesaurus, String type) {
            this.thesaurus = thesaurus;
            this.type = type;
        }

        @Override
        public Corpus getCorpus() {
            return thesaurus;
        }

        @Override
        public String getType() {
            return type;
        }

        @Override
        public int getFicheImportCount() {
            return motcleImportList.size();
        }

        @Override
        public FicheImport getFicheImport(int i) {
            return motcleImportList.get(i);
        }

        private void add(FicheImport ficheImport) {
            motcleImportList.add(ficheImport);
        }

    }


    private static class InternalFicheImport implements CorpusImport.FicheImport {

        private final FicheMeta ficheMeta;

        private InternalFicheImport(FicheMeta ficheMeta) {
            this.ficheMeta = ficheMeta;
        }

        @Override
        public FicheMeta getFicheMeta() {
            return ficheMeta;
        }

    }


    private static class InternalChangeFicheImport implements CorpusImport.ChangeFicheImport {

        private final FicheMeta ficheMeta;
        private final FicheChangeBuilder ficheChangeBuilder;
        private AttributeChangeBuilder attributeChangeBuilder;
        private LiensImportBuilder liensImportBuilder;
        private FicheChange ficheChange = null;
        private AttributeChange attributeChange = null;
        private LiensImport liensImport = null;
        private FuzzyDate creationDate = null;

        private InternalChangeFicheImport(FicheMeta ficheMeta, FicheChangeBuilder ficheChangeBuilder, AttributeChangeBuilder attributeChangeBuilder, LiensImportBuilder liensImportBuilder, FuzzyDate creationDate) {
            this.ficheMeta = ficheMeta;
            this.ficheChangeBuilder = ficheChangeBuilder;
            this.attributeChangeBuilder = attributeChangeBuilder;
            this.liensImportBuilder = liensImportBuilder;
            this.creationDate = creationDate;
        }

        @Override
        public FicheMeta getFicheMeta() {
            return ficheMeta;
        }

        @Override
        public FicheChange getFicheChange() {
            if (ficheChange == null) {
                ficheChange = ficheChangeBuilder.toFicheChange();
            }
            return ficheChange;
        }

        @Override
        public AttributeChange getAttributeChange() {
            if (attributeChange == null) {
                if (attributeChangeBuilder == null) {
                    attributeChange = AttributeUtils.EMPTY_ATTRIBUTECHANGE;
                } else {
                    attributeChange = attributeChangeBuilder.toAttributeChange();
                }
            }
            return attributeChange;
        }

        @Override
        public LiensImport getLiensImport() {
            if (liensImport == null) {
                if (liensImportBuilder == null) {
                    liensImport = ImportationUtils.EMPTY_LIENSIMPORT;
                } else {
                    liensImport = liensImportBuilder.toLiensImport();
                }
            }
            return liensImport;
        }

        @Override
        public FuzzyDate getCreationDate() {
            return creationDate;
        }

        private void merge(FicheChangeBuilder otherFicheChangeBuilder, AttributeChangeBuilder otherAttributeChangeBuilder, LiensImportBuilder otherLiensImportBuilder, FuzzyDate creationDate) {
            ficheChangeBuilder.merge(otherFicheChangeBuilder);
            if (otherAttributeChangeBuilder != null) {
                if (attributeChangeBuilder == null) {
                    attributeChangeBuilder = otherAttributeChangeBuilder;
                } else {
                    attributeChangeBuilder.merge(otherAttributeChangeBuilder);
                }
            }
            if (otherLiensImportBuilder != null) {
                if (liensImportBuilder == null) {
                    liensImportBuilder = otherLiensImportBuilder;
                } else {
                    liensImportBuilder.merge(otherLiensImportBuilder);
                }
            }
            if (this.creationDate == null) {
                this.creationDate = creationDate;
            }
        }

    }


    private static class InternalCreationFicheImport implements CorpusImport.CreationFicheImport {

        private final int newId;
        private final FicheChangeBuilder ficheChangeBuilder;
        private AttributeChangeBuilder attributeChangeBuilder;
        private LiensImportBuilder liensImportBuilder;
        private FicheChange ficheChange = null;
        private AttributeChange changeAttributeHolder = null;
        private LiensImport liensImport = null;
        private FuzzyDate creationDate = null;

        private InternalCreationFicheImport(int newId, FicheChangeBuilder ficheChangeBuilder, AttributeChangeBuilder attributeChangeBuilder, LiensImportBuilder liensImportBuilder, FuzzyDate creationDate) {
            this.newId = newId;
            this.ficheChangeBuilder = ficheChangeBuilder;
            this.attributeChangeBuilder = attributeChangeBuilder;
            this.liensImportBuilder = liensImportBuilder;
            this.creationDate = creationDate;
        }

        @Override
        public FicheMeta getFicheMeta() {
            return null;
        }

        @Override
        public int getNewId() {
            return newId;
        }

        @Override
        public FicheChange getFicheChange() {
            if (ficheChange == null) {
                ficheChange = ficheChangeBuilder.toFicheChange();
            }
            return ficheChange;
        }

        @Override
        public AttributeChange getAttributeChange() {
            if (changeAttributeHolder == null) {
                if (attributeChangeBuilder == null) {
                    changeAttributeHolder = AttributeUtils.EMPTY_ATTRIBUTECHANGE;
                } else {
                    changeAttributeHolder = attributeChangeBuilder.toAttributeChange();
                }
            }
            return changeAttributeHolder;
        }

        @Override
        public LiensImport getLiensImport() {
            if (liensImport == null) {
                if (liensImportBuilder == null) {
                    liensImport = ImportationUtils.EMPTY_LIENSIMPORT;
                } else {
                    liensImport = liensImportBuilder.toLiensImport();
                }
            }
            return liensImport;
        }

        @Override
        public FuzzyDate getCreationDate() {
            return creationDate;
        }

        private void merge(FicheChangeBuilder otherFicheChangeBuilder, AttributeChangeBuilder otherAttributeChangeBuilder, LiensImportBuilder otherLiensImportBuilder, FuzzyDate creationDate) {
            ficheChangeBuilder.merge(otherFicheChangeBuilder);
            if (otherAttributeChangeBuilder != null) {
                if (attributeChangeBuilder == null) {
                    attributeChangeBuilder = otherAttributeChangeBuilder;
                } else {
                    attributeChangeBuilder.merge(otherAttributeChangeBuilder);
                }
            }
            if (otherLiensImportBuilder != null) {
                if (liensImportBuilder == null) {
                    liensImportBuilder = otherLiensImportBuilder;
                } else {
                    liensImportBuilder.merge(otherLiensImportBuilder);
                }
            }
            if (this.creationDate == null) {
                this.creationDate = creationDate;
            }
        }

    }

}
